# -*- coding: utf-8 -*-
import pytest

import validators


@pytest.mark.parametrize('value', [
    'da39a3ee5e6b4b0d3255bfef95601890afd80709',
    'DA39A3EE5E6B4B0D3255BFEF95601890AFD80709'
])
def test_returns_true_on_valid_sha1(value):
    assert validators.sha1(value)


@pytest.mark.parametrize('value', [
    'za39a3ee5e6b4b0d3255bfef95601890afd80709',
    'da39e5e6b4b0d3255bfef95601890afd80709',
    'daaaa39a3ee5e6b4b0d3255bfef95601890afd80709'
])
def test_returns_failed_validation_on_invalid_sha1(value):
    result = validators.sha1(value)
    assert isinstance(result, validators.ValidationFailure)



from functools import total_ordering


@total_ordering
class Min(object):
    """
    An object that is less than any other object (except itself).

    Inspired by https://pypi.python.org/pypi/Extremes

    Examples::

        >>> import sys

        >>> Min < -sys.maxint
        True

        >>> Min < None
        True

        >>> Min < ''
        True

    .. versionadded:: 0.2
    """
    def __lt__(self, other):
        if other is Min:
            return False
        return True


@total_ordering
class Max(object):
    """
    An object that is greater than any other object (except itself).

    Inspired by https://pypi.python.org/pypi/Extremes

    Examples::

        >>> import sys

        >>> Max > Min
        True

        >>> Max > sys.maxint
        True

        >>> Max > 99999999999999999
        True

    .. versionadded:: 0.2
    """
    def __gt__(self, other):
        if other is Max:
            return False
        return True


Min = Min()
Max = Max()



import inspect
import itertools
from collections import OrderedDict

from decorator import decorator


class ValidationFailure(Exception):
    def __init__(self, func, args):
        self.func = func
        self.__dict__.update(args)

    def __repr__(self):
        return u'ValidationFailure(func={func}, args={args})'.format(
            func=self.func.__name__,
            args=dict(
                [(k, v) for (k, v) in self.__dict__.items() if k != 'func']
            )
        )

    def __str__(self):
        return repr(self)

    def __unicode__(self):
        return repr(self)

    def __bool__(self):
        return False

    def __nonzero__(self):
        return False


def func_args_as_dict(func, args, kwargs):
    """
    Return given function's positional and key value arguments as an ordered
    dictionary.
    """
    _getargspec = inspect.getfullargspec

    arg_names = list(
        OrderedDict.fromkeys(
            itertools.chain(
                _getargspec(func)[0],
                kwargs.keys()
            )
        )
    )
    return OrderedDict(
        list(zip(arg_names, args)) +
        list(kwargs.items())
    )


def validator(func, *args, **kwargs):
    """
    A decorator that makes given function validator.

    Whenever the given function is called and returns ``False`` value
    this decorator returns :class:`ValidationFailure` object.

    Example::

        >>> @validator
        ... def even(value):
        ...     return not (value % 2)

        >>> even(4)
        True

        >>> even(5)
        ValidationFailure(func=even, args={'value': 5})

    :param func: function to decorate
    :param args: positional function arguments
    :param kwargs: key value function arguments
    """
    def wrapper(func, *args, **kwargs):
        value = func(*args, **kwargs)
        if not value:
            return ValidationFailure(
                func, func_args_as_dict(func, args, kwargs)
            )
        return True
    return decorator(wrapper, func)



import validators

obj_repr = (
    "ValidationFailure(func=between"
)


class TestValidationFailure(object):
    def setup_method(self, method):
        self.obj = validators.between(3, min=4, max=5)

    def test_boolean_coerce(self):
        assert not bool(self.obj)
        assert not self.obj

    def test_repr(self):
        assert obj_repr in repr(self.obj)

    def test_unicode(self):
        assert obj_repr in str(self.obj)

    def test_arguments_as_properties(self):
        assert self.obj.value == 3
        assert self.obj.min == 4
        assert self.obj.max == 5



# -*- coding: utf-8 -*-
import pytest

from validators import ipv4_cidr, ipv6_cidr, ValidationFailure


@pytest.mark.parametrize(('cidr',), [
    ('::1/0',),
    ('dead:beef:0:0:0:0:42:1/8',),
    ('abcd:ef::42:1/32',),
    ('0:0:0:0:0:ffff:1.2.3.4/64',),
    ('::192.168.30.2/128',),
])
def test_returns_true_on_valid_ipv6_cidr(cidr):
    assert ipv6_cidr(cidr)
    assert not ipv4_cidr(cidr)


@pytest.mark.parametrize(('cidr',), [
    ('abc.0.0.1',),
    ('abcd:1234::123::1',),
    ('1:2:3:4:5:6:7:8:9',),
    ('abcd::1ffff',),
    ('1.1.1.1',),
    ('::1',),
    ('::1/129',),
    ('::1/-1',),
    ('::1/foo',),
])
def test_returns_failed_validation_on_invalid_ipv6_cidr(cidr):
    assert isinstance(ipv6_cidr(cidr), ValidationFailure)



# -*- coding: utf-8 -*-
from uuid import UUID

import pytest

from validators import uuid, ValidationFailure


@pytest.mark.parametrize(('value',), [
    ('2bc1c94f-0deb-43e9-92a1-4775189ec9f8',),
])
def test_returns_true_on_valid_mac_address(value):
    assert uuid(value)


@pytest.mark.parametrize(('value',), [
    (UUID('2bc1c94f-0deb-43e9-92a1-4775189ec9f8'),),
])
def test_returns_true_on_valid_uuid_object(value):
    assert uuid(value)


@pytest.mark.parametrize(('value',), [
    ('2bc1c94f-deb-43e9-92a1-4775189ec9f8',),
    ('2bc1c94f-0deb-43e9-92a1-4775189ec9f',),
    ('gbc1c94f-0deb-43e9-92a1-4775189ec9f8',),
    ('2bc1c94f 0deb-43e9-92a1-4775189ec9f8',),
])
def test_returns_failed_validation_on_invalid_mac_address(value):
    assert isinstance(uuid(value), ValidationFailure)


@pytest.mark.parametrize(('value',), [
    (1,),
    (1.0,),
    (True,),
    (None,),
])
def test_returns_failed_validation_on_invalid_types(value):
    assert isinstance(uuid(value), ValidationFailure)



# -*- coding: utf-8 -*-
import pytest

from validators import (
    amex,
    card_number,
    diners,
    discover,
    jcb,
    mastercard,
    unionpay,
    ValidationFailure,
    visa
)

visa_cards = [
    '4242424242424242',
    '4000002760003184'
]
mastercard_cards = [
    '5555555555554444',
    '2223003122003222'
]
amex_cards = [
    '378282246310005',
    '371449635398431'
]
unionpay_cards = [
    '6200000000000005'
]
diners_cards = [
    '3056930009020004',
    '36227206271667'
]
jcb_cards = [
    '3566002020360505'
]
discover_cards = [
    '6011111111111117',
    '6011000990139424'
]


@pytest.mark.parametrize(
    "value",
    visa_cards
    + mastercard_cards
    + amex_cards
    + unionpay_cards
    + diners_cards
    + jcb_cards
    + discover_cards,
)
def test_returns_true_on_valid_card_number(value):
    assert card_number(value)


@pytest.mark.parametrize('value', [
    '4242424242424240',
    '4000002760003180',
    '400000276000318X'
])
def test_returns_failed_on_valid_card_number(value):
    assert isinstance(card_number(value), ValidationFailure)


@pytest.mark.parametrize('value', visa_cards)
def test_returns_true_on_valid_visa(value):
    assert visa(value)


@pytest.mark.parametrize(
    "value",
    mastercard_cards
    + amex_cards
    + unionpay_cards
    + diners_cards
    + jcb_cards
    + discover_cards,
)
def test_returns_failed_on_valid_visa(value):
    assert isinstance(visa(value), ValidationFailure)


@pytest.mark.parametrize('value', mastercard_cards)
def test_returns_true_on_valid_mastercard(value):
    assert mastercard(value)


@pytest.mark.parametrize(
    "value",
    visa_cards
    + amex_cards
    + unionpay_cards
    + diners_cards
    + jcb_cards
    + discover_cards,
)
def test_returns_failed_on_valid_mastercard(value):
    assert isinstance(mastercard(value), ValidationFailure)


@pytest.mark.parametrize('value', amex_cards)
def test_returns_true_on_valid_amex(value):
    assert amex(value)


@pytest.mark.parametrize(
    "value",
    visa_cards
    + mastercard_cards
    + unionpay_cards
    + diners_cards
    + jcb_cards
    + discover_cards,
)
def test_returns_failed_on_valid_amex(value):
    assert isinstance(amex(value), ValidationFailure)


@pytest.mark.parametrize('value', unionpay_cards)
def test_returns_true_on_valid_unionpay(value):
    assert unionpay(value)


@pytest.mark.parametrize(
    "value",
    visa_cards
    + mastercard_cards
    + amex_cards
    + diners_cards
    + jcb_cards
    + discover_cards,
)
def test_returns_failed_on_valid_unionpay(value):
    assert isinstance(unionpay(value), ValidationFailure)


@pytest.mark.parametrize('value', diners_cards)
def test_returns_true_on_valid_diners(value):
    assert diners(value)


@pytest.mark.parametrize(
    "value",
    visa_cards
    + mastercard_cards
    + amex_cards
    + unionpay_cards
    + jcb_cards
    + discover_cards,
)
def test_returns_failed_on_valid_diners(value):
    assert isinstance(diners(value), ValidationFailure)


@pytest.mark.parametrize('value', jcb_cards)
def test_returns_true_on_valid_jcb(value):
    assert jcb(value)


@pytest.mark.parametrize(
    "value",
    visa_cards
    + mastercard_cards
    + amex_cards
    + unionpay_cards
    + diners_cards
    + discover_cards,
)
def test_returns_failed_on_valid_jcb(value):
    assert isinstance(jcb(value), ValidationFailure)


@pytest.mark.parametrize('value', discover_cards)
def test_returns_true_on_valid_discover(value):
    assert discover(value)


@pytest.mark.parametrize(
    "value",
    visa_cards
    + mastercard_cards
    + amex_cards
    + unionpay_cards
    + diners_cards
    + jcb_cards,
)
def test_returns_failed_on_valid_discover(value):
    assert isinstance(discover(value), ValidationFailure)



# -*- coding: utf-8 -*-
import pytest

import validators


@pytest.mark.parametrize(('value', 'min', 'max'), [
    ('password', 2, 10),
    ('password', None, 10),
    ('password', 2, None),
    ('password', 8, 8)
])
def test_returns_true_on_valid_length(value, min, max):
    assert validators.length(value, min=min, max=max)


@pytest.mark.parametrize(('value', 'min', 'max'), [
    ('something', 13, 12),
    ('something', -1, None),
    ('something', -1, None),
    ('something', -3, -2)
])
def test_raises_assertion_error_for_invalid_args(value, min, max):
    with pytest.raises(AssertionError):
        assert validators.length(value, min=min, max=max)


@pytest.mark.parametrize(('value', 'min', 'max'), [
    ('something', 13, 14),
    ('something', None, 6),
    ('something', 13, None)
])
def test_returns_failed_validation_on_invalid_range(value, min, max):
    assert isinstance(
        validators.length(value, min=min, max=max),
        validators.ValidationFailure
    )



from .between import between
from .btc_address import btc_address
from .card import (
    amex,
    card_number,
    diners,
    discover,
    jcb,
    mastercard,
    unionpay,
    visa
)
from .domain import domain
from .email import email
from .extremes import Max, Min
from .hashes import md5, sha1, sha224, sha256, sha512
from .i18n import fi_business_id, fi_ssn
from .iban import iban
from .ip_address import ipv4, ipv4_cidr, ipv6, ipv6_cidr
from .length import length
from .mac_address import mac_address
from .slug import slug
from .truthy import truthy
from .url import url
from .utils import ValidationFailure, validator
from .uuid import uuid

__all__ = ('between', 'domain', 'email', 'Max', 'Min', 'md5', 'sha1', 'sha224',
           'sha256', 'sha512', 'fi_business_id', 'fi_ssn', 'iban', 'ipv4',
           'ipv4_cidr', 'ipv6', 'ipv6_cidr', 'length', 'mac_address', 'slug',
           'truthy', 'url', 'ValidationFailure', 'validator', 'uuid',
           'card_number', 'visa', 'mastercard', 'amex', 'unionpay', 'diners',
           'jcb', 'discover', 'btc_address')

__version__ = '0.20.0'



# -*- coding: utf-8 -*-
import pytest

from validators import email, ValidationFailure


@pytest.mark.parametrize(('value', 'whitelist'), [
    ('email@here.com', None),
    ('weirder-email@here.and.there.com', None),
    ('email@[127.0.0.1]', None),
    ('example@valid-----hyphens.com', None),
    ('example@valid-with-hyphens.com', None),
    ('test@domain.with.idn.tld.उदाहरण.परीक्षा', None),
    ('email@localhost', None),
    ('email@localdomain', ['localdomain']),
    ('"test@test"@example.com', None),
    ('"\\\011"@here.com', None),
])
def test_returns_true_on_valid_email(value, whitelist):
    assert email(value, whitelist=whitelist)


@pytest.mark.parametrize(('value',), [
    (None,),
    ('',),
    ('abc',),
    ('abc@',),
    ('abc@bar',),
    ('a @x.cz',),
    ('abc@.com',),
    ('something@@somewhere.com',),
    ('email@127.0.0.1',),
    ('example@invalid-.com',),
    ('example@-invalid.com',),
    ('example@inv-.alid-.com',),
    ('example@inv-.-alid.com',),
    (
        'john56789.john56789.john56789.john56789.john56789.john56789.john5'
        '@example.com',
    ),
    # Quoted-string format (CR not allowed)
    ('"\\\012"@here.com',),
])
def test_returns_failed_validation_on_invalid_email(value):
    assert isinstance(email(value), ValidationFailure)



# -*- coding: utf-8 -*-
import pytest

import validators


@pytest.mark.parametrize('value', [
    'd41d8cd98f00b204e9800998ecf8427e',
    'D41D8CD98F00B204E9800998ECF8427E'
])
def test_returns_true_on_valid_md5(value):
    assert validators.md5(value)


@pytest.mark.parametrize('value', [
    'z41d8cd98f00b204e9800998ecf8427e',
    'z8cd98f00b204e9800998ecf8427e',
    'z4aaaa1d8cd98f00b204e9800998ecf8427e'
])
def test_returns_failed_validation_on_invalid_md5(value):
    result = validators.md5(value)
    assert isinstance(result, validators.ValidationFailure)



# -*- coding: utf-8 -*-
import pytest

import validators


@pytest.mark.parametrize(('value', 'min', 'max'), [
    (12, 11, 13),
    (12, None, 14),
    (12, 11, None),
    (12, 12, 12)
])
def test_returns_true_on_valid_range(value, min, max):
    assert validators.between(value, min=min, max=max)


@pytest.mark.parametrize(('value', 'min', 'max'), [
    (12, 13, 12),
    (12, None, None),
])
def test_raises_assertion_error_for_invalid_args(value, min, max):
    with pytest.raises(AssertionError):
        assert validators.between(value, min=min, max=max)


@pytest.mark.parametrize(('value', 'min', 'max'), [
    (12, 13, 14),
    (12, None, 11),
    (12, 13, None)
])
def test_returns_failed_validation_on_invalid_range(value, min, max):
    result = validators.between(value, min=min, max=max)
    assert isinstance(result, validators.ValidationFailure)



# -*- coding: utf-8 -*-
import pytest

from validators import ValidationFailure
from validators.i18n.fi import fi_business_id, fi_ssn


@pytest.mark.parametrize(('value',), [
    ('2336509-6',),  # Supercell
    ('0112038-9',),  # Fast Monkeys
    ('2417581-7',),  # Nokia
])
def test_returns_true_on_valid_business_id(value):
    assert fi_business_id(value)


@pytest.mark.parametrize(('value',), [
    (None,),
    ('',),
    ('1233312312',),
    ('1333333-8',),
    ('1231233-9',),
])
def test_returns_failed_validation_on_invalid_business_id(value):
    assert isinstance(fi_business_id(value), ValidationFailure)


@pytest.mark.parametrize(('value',), [
    ('010190-002R',),
    ('010101-0101',),
    ('010101+0101',),
    ('010101A0101',),
    ('010190-900P',),
])
def test_returns_true_on_valid_ssn(value):
    assert fi_ssn(value)


@pytest.mark.parametrize(('value',), [
    (None,),
    ('',),
    ('010190-001P',),  # Too low serial
    ('010190-000N',),  # Too low serial
    ('000190-0023',),  # Invalid day
    ('010090-002X',),  # Invalid month
    ('010190-002r',),  # Invalid checksum
    ('101010-0102',),
    ('10a010-0101',),
    ('101010-0\xe401',),
    ('101010b0101',)
])
def test_returns_failed_validation_on_invalid_ssn(value):
    assert isinstance(fi_ssn(value), ValidationFailure)


def test_returns_failed_validation_on_temporal_ssn_when_not_allowed():
    assert isinstance(
        fi_ssn('010190-900P', allow_temporal_ssn=False),
        ValidationFailure
    )



import re
from hashlib import sha256

from .utils import validator

segwit_pattern = re.compile(
    r'^(bc|tc)[0-3][02-9ac-hj-np-z]{14,74}$')


def validate_segwit_address(addr):
    return segwit_pattern.match(addr)


def decode_base58(addr):
    alphabet = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
    return sum([
        (58 ** e) * alphabet.index(i)
        for e, i in enumerate(addr[::-1])
    ])


def validate_old_btc_address(addr):
    "Validate P2PKH and P2SH type address"
    if not len(addr) in range(25, 35):
        return False
    decoded_bytes = decode_base58(addr).to_bytes(25, "big")
    header = decoded_bytes[:-4]
    checksum = decoded_bytes[-4:]
    return checksum == sha256(sha256(header).digest()).digest()[:4]


@validator
def btc_address(value):
    """
    Return whether or not given value is a valid bitcoin address.

    If the value is valid bitcoin address this function returns ``True``,
    otherwise :class:`~validators.utils.ValidationFailure`.

    Full validation is implemented for P2PKH and P2SH addresses.
    For segwit addresses a regexp is used to provide a reasonable estimate
    on whether the address is valid.

    Examples::

        >>> btc_address('3Cwgr2g7vsi1bXDUkpEnVoRLA9w4FZfC69')
        True

    :param value: Bitcoin address string to validate
    """
    if not value or not isinstance(value, str):
        return False
    if value[:2] in ("bc", "tb"):
        return validate_segwit_address(value)
    return validate_old_btc_address(value)



# -*- coding: utf-8 -*-
import pytest

from validators import url, ValidationFailure


@pytest.mark.parametrize('address', [
    u'http://foobar.dk',
    u'http://foobar.museum/foobar',
    u'http://fo.com',
    u'http://FOO.com',
    u'http://foo.com/blah_blah',
    u'http://foo.com/blah_blah/',
    u'http://foo.com/blah_blah_(wikipedia)',
    u'http://foo.com/blah_blah_(wikipedia)_(again)',
    u'http://www.example.com/wpstyle/?p=364',
    u'https://www.example.com/foo/?bar=baz&inga=42&quux',
    u'https://www.example.com?bar=baz',
    u'http://✪df.ws/123',
    u'http://userid:password@example.com:8080',
    u'http://userid:password@example.com:8080/',
    u'http://userid@example.com',
    u'http://userid@example.com/',
    u'http://userid@example.com:8080',
    u'http://userid@example.com:8080/',
    u'http://userid:password@example.com',
    u'http://userid:password@example.com/',
    u'http://142.42.1.1/',
    u'http://142.42.1.1:8080/',
    u'http://➡.ws/䨹',
    u'http://⌘.ws',
    u'http://⌘.ws/',
    u'http://foo.com/blah_(wikipedia)#cite-1',
    u'http://foo.com/blah_(wikipedia)_blah#cite-1',
    u'http://foo.com/unicode_(✪)_in_parens',
    u'http://foo.com/(something)?after=parens',
    u'http://☺.damowmow.com/',
    u'http://code.google.com/events/#&product=browser',
    u'http://j.mp',
    u'ftp://foo.bar/baz',
    u'http://foo.bar/?q=Test%20URL-encoded%20stuff',
    u'http://مثال.إختبار',
    u'http://例子.测试',
    u'http://उदाहरण.परीक्षा',
    u'http://www.😉.com',
    u'http://😉.com/😁',
    u'http://উদাহরণ.বাংলা',
    u'http://xn--d5b6ci4b4b3a.xn--54b7fta0cc',
    u'http://дом-м.рф/1/asdf',
    u'http://xn----gtbybh.xn--p1ai/1/asdf',
    u'http://-.~_!$&\'()*+,;=:%40:80%2f::::::@example.com',
    u'http://1337.net',
    u'http://a.b-c.de',
    u'http://223.255.255.254',
    u'http://10.1.1.0',
    u'http://10.1.1.1',
    u'http://10.1.1.254',
    u'http://10.1.1.255',
    u'http://127.0.0.1:8080',
    u'http://127.0.10.150',
    u'http://localhost',
    u'http://localhost:8000',
    u'http://[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]:80/index.html',
    u'http://[1080:0:0:0:8:800:200C:417A]/index.html',
    u'http://[3ffe:2a00:100:7031::1]',
    u'http://[1080::8:800:200C:417A]/foo',
    u'http://[::192.9.5.5]/ipng',
    u'http://[::FFFF:129.144.52.38]:80/index.html',
    u'http://[2010:836B:4179::836B:4179]',
])
def test_returns_true_on_valid_url(address):
    assert url(address)


@pytest.mark.parametrize('address, public', [
    (u'http://foo.bar', True),
    (u'http://username:password@example.com:4010/', False),
    (u'http://username:password@112.168.10.10:4010/', True),
    (u'http://username:password@192.168.10.10:4010/', False),
    (u'http://10.0.10.1', False),
    (u'http://127.0.0.1', False),
])
def test_returns_true_on_valid_public_url(address, public):
    assert url(address, public=public)


@pytest.mark.parametrize('address', [
    'http://foobar',
    'foobar.dk',
    'http://127.0.0/asdf',
    'http://foobar.d',
    'http://foobar.12',
    'http://foobar',
    'htp://foobar.com',
    'http://foobar..com',
    'http://fo..com',
    'http://',
    'http://.',
    'http://..',
    'http://../',
    'http://?',
    'http://??',
    'http://??/',
    'http://#',
    'http://##',
    'http://##/',
    'http://foo.bar?q=Spaces should be encoded',
    '//',
    '//a',
    '///a',
    '///',
    'http:///a',
    'foo.com',
    'rdar://1234',
    'h://test',
    'http:// shouldfail.com',
    ':// should fail',
    'http://foo.bar/foo(bar)baz quux',
    'ftps://foo.bar/',
    'http://-error-.invalid/',
    'http://a.b--c.de/',
    'http://-a.b.co',
    'http://a.b-.co',
    'http://0.0.0.0',
    'http://224.1.1.1',
    'http://1.1.1.1.1',
    'http://123.123.123',
    'http://3628126748',
    'http://.www.foo.bar/',
    'http://www.foo.bar./',
    'http://.www.foo.bar./',
    'http://127.12.0.260',
    'http://example.com/">user@example.com',
    'http://[2010:836B:4179::836B:4179',
    'http://2010:836B:4179::836B:4179',
    'http://2010:836B:4179::836B:4179:80/index.html',
])
def test_returns_failed_validation_on_invalid_url(address):
    assert isinstance(url(address), ValidationFailure)


@pytest.mark.parametrize('address, public', [
    (u'http://username:password@192.168.10.10:4010/', True),
    (u'http://10.0.10.1', True),
    (u'http://127.0.0.1', True),
    (u'foo://127.0.0.1', True),
    (u'http://username:password@127.0.0.1:8080', True),
    (u'http://localhost', True),
    (u'http://localhost:8000', True),

])
def test_returns_failed_validation_on_invalid_public_url(address, public):
    assert isinstance(url(address, public=public), ValidationFailure)



# -*- coding: utf-8 -*-
import pytest

from validators import ipv4, ipv6, ValidationFailure


@pytest.mark.parametrize(('address',), [
    ('127.0.0.1',),
    ('123.5.77.88',),
    ('12.12.12.12',),
])
def test_returns_true_on_valid_ipv4_address(address):
    assert ipv4(address)
    assert not ipv6(address)


@pytest.mark.parametrize(('address',), [
    ('abc.0.0.1',),
    ('1278.0.0.1',),
    ('127.0.0.abc',),
    ('900.200.100.75',),
    ('0127.0.0.1',),
])
def test_returns_failed_validation_on_invalid_ipv4_address(address):
    assert isinstance(ipv4(address), ValidationFailure)



import re

from .utils import validator

md5_regex = re.compile(
    r"^[0-9a-f]{32}$",
    re.IGNORECASE
)
sha1_regex = re.compile(
    r"^[0-9a-f]{40}$",
    re.IGNORECASE
)
sha224_regex = re.compile(
    r"^[0-9a-f]{56}$",
    re.IGNORECASE
)
sha256_regex = re.compile(
    r"^[0-9a-f]{64}$",
    re.IGNORECASE
)
sha512_regex = re.compile(
    r"^[0-9a-f]{128}$",
    re.IGNORECASE
)


@validator
def md5(value):
    """
    Return whether or not given value is a valid MD5 hash.

    Examples::

        >>> md5('d41d8cd98f00b204e9800998ecf8427e')
        True

        >>> md5('900zz11')
        ValidationFailure(func=md5, args={'value': '900zz11'})

    :param value: MD5 string to validate
    """
    return md5_regex.match(value)


@validator
def sha1(value):
    """
    Return whether or not given value is a valid SHA1 hash.

    Examples::

        >>> sha1('da39a3ee5e6b4b0d3255bfef95601890afd80709')
        True

        >>> sha1('900zz11')
        ValidationFailure(func=sha1, args={'value': '900zz11'})

    :param value: SHA1 string to validate
    """
    return sha1_regex.match(value)


@validator
def sha224(value):
    """
    Return whether or not given value is a valid SHA224 hash.

    Examples::

        >>> sha224('d14a028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f')
        True

        >>> sha224('900zz11')
        ValidationFailure(func=sha224, args={'value': '900zz11'})

    :param value: SHA224 string to validate
    """
    return sha224_regex.match(value)


@validator
def sha256(value):
    """
    Return whether or not given value is a valid SHA256 hash.

    Examples::

        >>> sha256(
        ...     'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b'
        ...     '855'
        ... )
        True

        >>> sha256('900zz11')
        ValidationFailure(func=sha256, args={'value': '900zz11'})

    :param value: SHA256 string to validate
    """
    return sha256_regex.match(value)


@validator
def sha512(value):
    """
    Return whether or not given value is a valid SHA512 hash.

    Examples::

        >>> sha512(
        ...     'cf83e1357eefb8bdf1542850d66d8007d620e4050b5715dc83f4a921d36ce'
        ...     '9ce47d0d13c5d85f2b0ff8318d2877eec2f63b931bd47417a81a538327af9'
        ...     '27da3e'
        ... )
        True

        >>> sha512('900zz11')
        ValidationFailure(func=sha512, args={'value': '900zz11'})

    :param value: SHA512 string to validate
    """
    return sha512_regex.match(value)



from __future__ import absolute_import

import re
from uuid import UUID

from .utils import validator

pattern = re.compile(r'^[0-9a-fA-F]{8}-([0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}$')


@validator
def uuid(value):
    """
    Return whether or not given value is a valid UUID.

    If the value is valid UUID this function returns ``True``, otherwise
    :class:`~validators.utils.ValidationFailure`.

    This validator is based on `WTForms UUID validator`_.

    .. _WTForms UUID validator:
       https://github.com/wtforms/wtforms/blob/master/wtforms/validators.py

    Examples::

        >>> uuid('2bc1c94f-0deb-43e9-92a1-4775189ec9f8')
        True

        >>> uuid('2bc1c94f 0deb-43e9-92a1-4775189ec9f8')
        ValidationFailure(func=uuid, ...)

    .. versionadded:: 0.2

    :param value: UUID value to validate
    """
    if isinstance(value, UUID):
        return True
    try:
        return pattern.match(value)
    except TypeError:
        return False



# -*- coding: utf-8 -*-
import pytest

import validators


@pytest.mark.parametrize('value', [
    'd14a028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f',
    'D14A028C2A3A2BC9476102BB288234C415A2B01F828EA62AC5B3E42F'
])
def test_returns_true_on_valid_sha224(value):
    assert validators.sha224(value)


@pytest.mark.parametrize('value', [
    'z14a028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f',
    'd028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f',
    'daaa14a028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f'
])
def test_returns_failed_validation_on_invalid_sha224(value):
    result = validators.sha224(value)
    assert isinstance(result, validators.ValidationFailure)



# -*- coding: utf-8 -*-
import pytest

import validators


@pytest.mark.parametrize('value', [
    'GB82WEST12345698765432',
    'NO9386011117947'
])
def test_returns_true_on_valid_iban(value):
    assert validators.iban(value)


@pytest.mark.parametrize('value', [
    'GB81WEST12345698765432',
    'NO9186011117947'
])
def test_returns_failed_validation_on_invalid_iban(value):
    result = validators.iban(value)
    assert isinstance(result, validators.ValidationFailure)



# -*- coding: utf-8 -*-
import pytest

from validators import Max, Min


@pytest.mark.parametrize(('value',), [
    (None,),
    ('',),
    (12,),
    (Min,),
])
def test_max_is_greater_than_every_other_value(value):
    assert value < Max
    assert Max > value


def test_max_is_not_greater_than_itself():
    assert not (Max < Max)


def test_other_comparison_methods_for_max():
    assert Max <= Max
    assert Max == Max
    assert not (Max != Max)


@pytest.mark.parametrize(('value',), [
    (None,),
    ('',),
    (12,),
    (Max,),
])
def test_min_is_smaller_than_every_other_value(value):
    assert value > Min


def test_min_is_not_greater_than_itself():
    assert not (Min < Min)


def test_other_comparison_methods_for_min():
    assert Min <= Min
    assert Min == Min
    assert not (Min != Min)



import re

from .utils import validator

pattern = re.compile(
    r'^(?:[a-zA-Z0-9]'  # First character of the domain
    r'(?:[a-zA-Z0-9-_]{0,61}[A-Za-z0-9])?\.)'  # Sub domain + hostname
    r'+[A-Za-z0-9][A-Za-z0-9-_]{0,61}'  # First 61 characters of the gTLD
    r'[A-Za-z]$'  # Last character of the gTLD
)


def to_unicode(obj, charset='utf-8', errors='strict'):
    if obj is None:
        return None
    if not isinstance(obj, bytes):
        return str(obj)
    return obj.decode(charset, errors)


@validator
def domain(value):
    """
    Return whether or not given value is a valid domain.

    If the value is valid domain name this function returns ``True``, otherwise
    :class:`~validators.utils.ValidationFailure`.

    Examples::

        >>> domain('example.com')
        True

        >>> domain('example.com/')
        ValidationFailure(func=domain, ...)


    Supports IDN domains as well::

        >>> domain('xn----gtbspbbmkef.xn--p1ai')
        True

    .. versionadded:: 0.9

    .. versionchanged:: 0.10

        Added support for internationalized domain name (IDN) validation.

    :param value: domain string to validate
    """
    try:
        return pattern.match(to_unicode(value).encode('idna').decode('ascii'))
    except (UnicodeError, AttributeError):
        return False



import re

from .utils import validator

slug_regex = re.compile(r'^[-a-zA-Z0-9_]+$')


@validator
def slug(value):
    """
    Validate whether or not given value is valid slug.

    Valid slug can contain only alphanumeric characters, hyphens and
    underscores.

    Examples::

        >>> slug('my.slug')
        ValidationFailure(func=slug, args={'value': 'my.slug'})

        >>> slug('my-slug-2134')
        True

    .. versionadded:: 0.6

    :param value: value to validate
    """
    return slug_regex.match(value)



# -*- coding: utf-8 -*-
import pytest

from validators import btc_address, ValidationFailure


@pytest.mark.parametrize(
    'value',
    [
        # P2PKH (Pay-to-PubkeyHash) type
        '1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2',
        # P2SH (Pay to script hash) type
        '3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy',
        # Bech32/segwit type
        'bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq',
        'bc1qc7slrfxkknqcq2jevvvkdgvrt8080852dfjewde450xdlk4ugp7szw5tk9',
    ],
)
def test_returns_true_on_valid_btc_address(value):
    assert btc_address(value)


@pytest.mark.parametrize(
    'value',
    [
        'ff3Cwgr2g7vsi1bXDUkpEnVoRLA9w4FZfC69',
        'b3Cgwgr2g7vsi1bXyjyDUkphEnVoRLA9w4FZfC69',
        # incorrect header
        '1BvBMsEYstWetqTFn5Au4m4GFg7xJaNVN2',
        # incorrect checksum
        '3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLz',
    ],
)
def test_returns_failed_validation_on_invalid_btc_address(value):
    assert isinstance(btc_address(value), ValidationFailure)



# -*- coding: utf-8 -*-
import pytest

from validators import ipv4_cidr, ipv6_cidr, ValidationFailure


@pytest.mark.parametrize(('cidr',), [
    ('127.0.0.1/0',),
    ('123.5.77.88/8',),
    ('12.12.12.12/32',),
])
def test_returns_true_on_valid_ipv4_cidr(cidr):
    assert ipv4_cidr(cidr)
    assert not ipv6_cidr(cidr)


@pytest.mark.parametrize(('cidr',), [
    ('abc.0.0.1',),
    ('1.1.1.1',),
    ('1.1.1.1/-1',),
    ('1.1.1.1/33',),
    ('1.1.1.1/foo',),
])
def test_returns_failed_validation_on_invalid_ipv4_cidr(cidr):
    assert isinstance(ipv4_cidr(cidr), ValidationFailure)



# -*- coding: utf-8 -*-
import pytest

from validators import domain, ValidationFailure


@pytest.mark.parametrize('value', [
    'example.com',
    'xn----gtbspbbmkef.xn--p1ai',
    'underscore_subdomain.example.com',
    'something.versicherung',
    'someThing.versicherung',
    '11.com',
    '3.cn',
    'a.cn',
    'sub1.sub2.sample.co.uk',
    'somerandomexample.xn--fiqs8s',
    'kräuter.com',
    'über.com'
])
def test_returns_true_on_valid_domain(value):
    assert domain(value)


@pytest.mark.parametrize('value', [
    'example.com/',
    'example.com:4444',
    'example.-com',
    'example.',
    '-example.com',
    'example-.com',
    '_example.com',
    'example_.com',
    'example',
    'a......b.com',
    'a.123',
    '123.123',
    '123.123.123',
    '123.123.123.123'
])
def test_returns_failed_validation_on_invalid_domain(value):
    assert isinstance(domain(value), ValidationFailure)



# -*- coding: utf-8 -*-
"""
validators
----------

Python Data Validation for Humans™.
"""

from setuptools import setup, find_packages
import os
import re
import sys


PY3 = sys.version_info[0] == 3
HERE = os.path.dirname(os.path.abspath(__file__))


def get_version():
    filename = os.path.join(HERE, 'validators', '__init__.py')
    with open(filename) as f:
        contents = f.read()
    pattern = r"^__version__ = '(.*?)'$"
    return re.search(pattern, contents, re.MULTILINE).group(1)


extras_require = {
    'test': [
        'pytest>=2.2.3',
        'flake8>=2.4.0',
        'isort>=4.2.2'
    ],
}

install_requires = [
    'decorator>=3.4.0',
]

setup(
    name='validators',
    version=get_version(),
    url='https://github.com/kvesteri/validators',
    license='MIT',
    author='Konsta Vesterinen',
    author_email='konsta@fastmonkeys.com',
    description='Python Data Validation for Humans™.',
    long_description=__doc__,
    packages=find_packages('.', exclude=['tests', 'tests.*']),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=install_requires,
    build_requires=install_requires,
    extras_require=extras_require,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    python_requires='>=3.4'
)



# -*- coding: utf-8 -*-
import pytest

from validators import slug, ValidationFailure


@pytest.mark.parametrize('value', [
    '123-12312-asdasda',
    '123____123',
    'dsadasd-dsadas',
])
def test_returns_true_on_valid_slug(value):
    assert slug(value)


@pytest.mark.parametrize('value', [
    'some.slug',
    '1231321%',
    '   21312',
    '123asda&',
])
def test_returns_failed_validation_on_invalid_slug(value):
    assert isinstance(slug(value), ValidationFailure)



# -*- coding: utf-8 -*-
import pytest

from validators import mac_address, ValidationFailure


@pytest.mark.parametrize(('address',), [
    ('01:23:45:67:ab:CD',),
])
def test_returns_true_on_valid_mac_address(address):
    assert mac_address(address)


@pytest.mark.parametrize(('address',), [
    ('00:00:00:00:00',),
    ('01:23:45:67:89:',),
    ('01:23:45:67:89:gh',),
    ('123:23:45:67:89:00',),
])
def test_returns_failed_validation_on_invalid_mac_address(address):
    assert isinstance(mac_address(address), ValidationFailure)



from .between import between
from .utils import validator


@validator
def length(value, min=None, max=None):
    """
    Return whether or not the length of given string is within a specified
    range.

    Examples::

        >>> length('something', min=2)
        True

        >>> length('something', min=9, max=9)
        True

        >>> length('something', max=5)
        ValidationFailure(func=length, ...)

    :param value:
        The string to validate.
    :param min:
        The minimum required length of the string. If not provided, minimum
        length will not be checked.
    :param max:
        The maximum length of the string. If not provided, maximum length
        will not be checked.

    .. versionadded:: 0.2
    """
    if (min is not None and min < 0) or (max is not None and max < 0):
        raise AssertionError(
            '`min` and `max` need to be greater than zero.'
        )
    return between(len(value), min=min, max=max)



from .extremes import Max, Min
from .utils import validator


@validator
def between(value, min=None, max=None):
    """
    Validate that a number is between minimum and/or maximum value.

    This will work with any comparable type, such as floats, decimals and dates
    not just integers.

    This validator is originally based on `WTForms NumberRange validator`_.

    .. _WTForms NumberRange validator:
       https://github.com/wtforms/wtforms/blob/master/wtforms/validators.py

    Examples::

        >>> from datetime import datetime

        >>> between(5, min=2)
        True

        >>> between(13.2, min=13, max=14)
        True

        >>> between(500, max=400)
        ValidationFailure(func=between, args=...)

        >>> between(
        ...     datetime(2000, 11, 11),
        ...     min=datetime(1999, 11, 11)
        ... )
        True

    :param min:
        The minimum required value of the number. If not provided, minimum
        value will not be checked.
    :param max:
        The maximum value of the number. If not provided, maximum value
        will not be checked.

    .. versionadded:: 0.2
    """
    if min is None and max is None:
        raise AssertionError(
            'At least one of `min` or `max` must be specified.'
        )
    if min is None:
        min = Min
    if max is None:
        max = Max
    try:
        min_gt_max = min > max
    except TypeError:
        min_gt_max = max < min
    if min_gt_max:
        raise AssertionError('`min` cannot be more than `max`.')

    return min <= value and max >= value



# -*- coding: utf-8 -*-
import pytest

import validators


@pytest.mark.parametrize('value', [
    'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855',
    'E3B0C44298FC1C149AFBF4C8996FB92427AE41E4649B934CA495991B7852B855'
])
def test_returns_true_on_valid_sha256(value):
    assert validators.sha256(value)


@pytest.mark.parametrize('value', [
    'z3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855',
    'ec44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855',
    'eaaaa3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855',
])
def test_returns_failed_validation_on_invalid_sha256(value):
    result = validators.sha256(value)
    assert isinstance(result, validators.ValidationFailure)



# -*- coding: utf-8 -*-
from validators.utils import validator

__all__ = ('es_cif', 'es_nif', 'es_nie', 'es_doi',)


def nif_nie_validation(doi, number_by_letter, special_cases):
    """
    Validate if the doi is a NIF or a NIE.
    :param doi: DOI to validate.
    :return: boolean if it's valid.
    """
    doi = doi.upper()
    if doi in special_cases:
        return False

    table = 'TRWAGMYFPDXBNJZSQVHLCKE'

    if len(doi) != 9:
        return False

    control = doi[8]

    # If it is not a DNI, convert the first letter to the corresponding
    # digit
    numbers = number_by_letter.get(doi[0], doi[0]) + doi[1:8]

    return numbers.isdigit() and control == table[int(numbers) % 23]


@validator
def es_cif(doi):
    """
    Validate a Spanish CIF.

    Each company in Spain prior to 2008 had a distinct CIF and has been
    discontinued. For more information see `wikipedia.org/cif`_.

    The new replacement is to use NIF for absolutely everything. The issue is
    that there are "types" of NIFs now: company, person[citizen vs recident]
    all distinguished by the first character of the DOI. For this reason we
    will continue to call CIF NIFs that are used for companies.

    This validator is based on `generadordni.es`_.

    .. _generadordni.es:
        https://generadordni.es/

    .. _wikipedia.org/cif:
        https://es.wikipedia.org/wiki/C%C3%B3digo_de_identificaci%C3%B3n_fiscal

    Examples::

        >>> es_cif('B25162520')
        True

        >>> es_cif('B25162529')
        ValidationFailure(func=es_cif, args=...)

    .. versionadded:: 0.13.0

    :param doi: DOI to validate
    """
    doi = doi.upper()

    if len(doi) != 9:
        return False

    table = 'JABCDEFGHI'
    first_chr = doi[0]
    doi_body = doi[1:8]
    control = doi[8]

    if not doi_body.isdigit():
        return False

    odd_result = 0
    even_result = 0
    for index, char in enumerate(doi_body):
        if index % 2 == 0:
            # Multiply each each odd position doi digit by 2 and sum it all
            # together
            odd_result += sum(map(int, str(int(char) * 2)))
        else:
            even_result += int(char)

    res = (10 - (even_result + odd_result) % 10) % 10

    if first_chr in 'ABEH':  # Number type
        return str(res) == control
    elif first_chr in 'PSQW':  # Letter type
        return table[res] == control
    elif first_chr not in 'CDFGJNRUV':
        return False

    return control == str(res) or control == table[res]


@validator
def es_nif(doi):
    """
    Validate a Spanish NIF.

    Each entity, be it person or company in Spain has a distinct NIF. Since
    we've designated CIF to be a company NIF, this NIF is only for person.
    For more information see `wikipedia.org/nif`_.

    This validator is based on `generadordni.es`_.

    .. _generadordni.es:
        https://generadordni.es/

    .. _wikipedia.org/nif:
        https://es.wikipedia.org/wiki/N%C3%BAmero_de_identificaci%C3%B3n_fiscal

    Examples::

        >>> es_nif('26643189N')
        True

        >>> es_nif('26643189X')
        ValidationFailure(func=es_nif, args=...)

    .. versionadded:: 0.13.0

    :param doi: DOI to validate
    """
    number_by_letter = {'L': '0', 'M': '0', 'K': '0'}
    special_cases = ['X0000000T', '00000000T', '00000001R']
    return nif_nie_validation(doi, number_by_letter, special_cases)


@validator
def es_nie(doi):
    """
    Validate a Spanish NIE.

    The NIE is a tax identification number in Spain, known in Spanish as the
    NIE, or more formally the Número de identidad de extranjero. For more
    information see `wikipedia.org/nie`_.

    This validator is based on `generadordni.es`_.

    .. _generadordni.es:
        https://generadordni.es/

    .. _wikipedia.org/nie:
        https://es.wikipedia.org/wiki/N%C3%BAmero_de_identidad_de_extranjero

    Examples::

        >>> es_nie('X0095892M')
        True

        >>> es_nie('X0095892X')
        ValidationFailure(func=es_nie, args=...)

    .. versionadded:: 0.13.0

    :param doi: DOI to validate
    """
    number_by_letter = {'X': '0', 'Y': '1', 'Z': '2'}
    special_cases = ['X0000000T']

    # NIE must must start with X Y or Z
    if not doi or doi[0] not in number_by_letter.keys():
        return False

    return nif_nie_validation(doi, number_by_letter, special_cases)


@validator
def es_doi(doi):
    """
    Validate a Spanish DOI.

    A DOI in spain is all NIF / CIF / NIE / DNI -- a digital ID. For more
    information see `wikipedia.org/doi`_.

    This validator is based on `generadordni.es`_.

    .. _generadordni.es:
        https://generadordni.es/

    .. _wikipedia.org/doi:
        https://es.wikipedia.org/wiki/Identificador_de_objeto_digital

    Examples::

        >>> es_doi('X0095892M')
        True

        >>> es_doi('X0095892X')
        ValidationFailure(func=es_doi, args=...)

    .. versionadded:: 0.13.0

    :param doi: DOI to validate
    """
    return es_nie(doi) or es_nif(doi) or es_cif(doi)
